@@ -105,6 +105,7 @@ QT_BEGIN_NAMESPACE
     \qmlsignal QtQuick.Controls::AbstractButton::pressAndHold()
 
     This signal is emitted when the button is interactively pressed and held down by the user via touch or mouse.
+    It is not emitted when \l autoRepeat is enabled.
 */
 
 /*!
@@ -201,10 +202,17 @@ void QQuickAbstractButtonPrivate::handleUngrab()
     emit q->canceled();
 }
 
+bool QQuickAbstractButtonPrivate::acceptKeyClick(Qt::Key key) const
+{
+    return key == Qt::Key_Space;
+}
+
 bool QQuickAbstractButtonPrivate::isPressAndHoldConnected()
 {
     Q_Q(QQuickAbstractButton);
-    IS_SIGNAL_CONNECTED(q, QQuickAbstractButton, pressAndHold, ());
+    const auto signal = &QQuickAbstractButton::pressAndHold;
+    const QMetaMethod method = QMetaMethod::fromSignal(signal);
+    return q->isSignalConnected(method);
 }
 
 void QQuickAbstractButtonPrivate::startPressAndHold()
@@ -322,9 +330,10 @@ void QQuickAbstractButtonPrivate::click()
 void QQuickAbstractButtonPrivate::trigger()
 {
     Q_Q(QQuickAbstractButton);
+    const bool wasEnabled = effectiveEnable;
     if (action && action->isEnabled())
         QQuickActionPrivate::get(action)->trigger(q, false);
-    else if (effectiveEnable)
+    if (wasEnabled && (!action || !action->isEnabled()))
         emit q->clicked();
 }
 
@@ -357,6 +366,22 @@ void QQuickAbstractButtonPrivate::executeIndicator(bool complete)
         quickCompleteDeferred(q, indicatorName(), indicator);
 }
 
+void QQuickAbstractButtonPrivate::itemImplicitWidthChanged(QQuickItem *item)
+{
+    Q_Q(QQuickAbstractButton);
+    QQuickControlPrivate::itemImplicitWidthChanged(item);
+    if (item == indicator)
+        emit q->implicitIndicatorWidthChanged();
+}
+
+void QQuickAbstractButtonPrivate::itemImplicitHeightChanged(QQuickItem *item)
+{
+    Q_Q(QQuickAbstractButton);
+    QQuickControlPrivate::itemImplicitHeightChanged(item);
+    if (item == indicator)
+        emit q->implicitIndicatorHeightChanged();
+}
+
 QQuickAbstractButton *QQuickAbstractButtonPrivate::findCheckedButton() const
 {
     Q_Q(const QQuickAbstractButton);
@@ -427,9 +452,12 @@ QQuickAbstractButton::QQuickAbstractButton(QQuickAbstractButtonPrivate &dd, QQui
 QQuickAbstractButton::~QQuickAbstractButton()
 {
     Q_D(QQuickAbstractButton);
+    d->removeImplicitSizeListener(d->indicator);
     if (d->group)
         d->group->removeButton(this);
+#if QT_CONFIG(shortcut)
     d->ungrabShortcut();
+#endif
 }
 
 /*!
@@ -631,6 +659,9 @@ void QQuickAbstractButton::setAutoExclusive(bool exclusive)
     This property holds whether the button repeats \l pressed(), \l released()
     and \l clicked() signals while the button is pressed and held down.
 
+    If this property is set to \c true, the \l pressAndHold() signal will not
+    be emitted.
+
     The default value is \c false.
 
     The initial delay and the repetition interval are defined in milliseconds
@@ -675,13 +706,24 @@ void QQuickAbstractButton::setIndicator(QQuickItem *indicator)
     if (!d->indicator.isExecuting())
         d->cancelIndicator();
 
+    const qreal oldImplicitIndicatorWidth = implicitIndicatorWidth();
+    const qreal oldImplicitIndicatorHeight = implicitIndicatorHeight();
+
+    d->removeImplicitSizeListener(d->indicator);
     delete d->indicator;
     d->indicator = indicator;
+
     if (indicator) {
         if (!indicator->parentItem())
             indicator->setParentItem(this);
         indicator->setAcceptedMouseButtons(Qt::LeftButton);
+        d->addImplicitSizeListener(indicator);
     }
+
+    if (!qFuzzyCompare(oldImplicitIndicatorWidth, implicitIndicatorWidth()))
+        emit implicitIndicatorWidthChanged();
+    if (!qFuzzyCompare(oldImplicitIndicatorHeight, implicitIndicatorHeight()))
+        emit implicitIndicatorHeightChanged();
     if (!d->indicator.isExecuting())
         emit indicatorChanged();
 }
@@ -698,7 +740,7 @@ void QQuickAbstractButton::setIndicator(QQuickItem *indicator)
 
     \include qquickicon.qdocinc grouped-properties
 
-    \sa text, display, {Icons in Qt Quick Controls 2}
+    \sa text, display, {Icons in Qt Quick Controls}
 */
 
 QQuickIcon QQuickAbstractButton::icon() const
@@ -726,6 +768,7 @@ void QQuickAbstractButton::setIcon(const QQuickIcon &icon)
     \row \li \c AbstractButton.IconOnly \li \image qtquickcontrols2-button-icononly.png
     \row \li \c AbstractButton.TextOnly \li \image qtquickcontrols2-button-textonly.png
     \row \li \c AbstractButton.TextBesideIcon \li \image qtquickcontrols2-button-textbesideicon.png
+    \row \li \c AbstractButton.TextUnderIcon \li \image qtquickcontrols2-button-textundericon.png
     \endtable
 
     \sa {Control::}{spacing}, {Control::}{padding}
@@ -908,6 +951,50 @@ qreal QQuickAbstractButton::pressY() const
     return d->movePoint.y();
 }
 
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::AbstractButton::implicitIndicatorWidth
+    \readonly
+
+    This property holds the implicit indicator width.
+
+    The value is equal to \c {indicator ? indicator.implicitWidth : 0}.
+
+    This is typically used, together with \l {Control::}{implicitContentWidth} and
+    \l {Control::}{implicitBackgroundWidth}, to calculate the \l {Item::}{implicitWidth}.
+
+    \sa implicitIndicatorHeight
+*/
+qreal QQuickAbstractButton::implicitIndicatorWidth() const
+{
+    Q_D(const QQuickAbstractButton);
+    if (!d->indicator)
+        return 0;
+    return d->indicator->implicitWidth();
+}
+
+/*!
+    \since QtQuick.Controls 2.5 (Qt 5.12)
+    \qmlproperty real QtQuick.Controls::AbstractButton::implicitIndicatorHeight
+    \readonly
+
+    This property holds the implicit indicator height.
+
+    The value is equal to \c {indicator ? indicator.implicitHeight : 0}.
+
+    This is typically used, together with \l {Control::}{implicitContentHeight} and
+    \l {Control::}{implicitBackgroundHeight}, to calculate the \l {Item::}{implicitHeight}.
+
+    \sa implicitIndicatorWidth
+*/
+qreal QQuickAbstractButton::implicitIndicatorHeight() const
+{
+    Q_D(const QQuickAbstractButton);
+    if (!d->indicator)
+        return 0;
+    return d->indicator->implicitHeight();
+}
+
 /*!
     \qmlmethod void QtQuick.Controls::AbstractButton::toggle()
 
@@ -953,7 +1040,7 @@ void QQuickAbstractButton::keyPressEvent(QKeyEvent *event)
 {
     Q_D(QQuickAbstractButton);
     QQuickControl::keyPressEvent(event);
-    if (event->key() == Qt::Key_Space) {
+    if (d->acceptKeyClick(static_cast<Qt::Key>(event->key()))) {
         d->setPressPoint(QPoint(qRound(width() / 2), qRound(height() / 2)));
         setPressed(true);
 
@@ -969,7 +1056,7 @@ void QQuickAbstractButton::keyReleaseEvent(QKeyEvent *event)
 {
     Q_D(QQuickAbstractButton);
     QQuickControl::keyReleaseEvent(event);
-    if (event->key() == Qt::Key_Space) {
+    if (d->acceptKeyClick(static_cast<Qt::Key>(event->key()))) {
         setPressed(false);
 
         nextCheckState();
@@ -1074,6 +1161,10 @@ void QQuickAbstractButton::accessibilityActiveChanged(bool active)
 
 QAccessible::Role QQuickAbstractButton::accessibleRole() const
 {
+    Q_D(const QQuickAbstractButton);
+    if (d->checkable) {
+        return QAccessible::CheckBox;
+    }
     return QAccessible::Button;
 }
 #endif
