@@ -41,8 +41,10 @@
 #include "../shared/qtest_quickcontrols.h"
 
 #include <QtGui/qpa/qwindowsysteminterface.h>
+#include <QtQuick/qquickview.h>
 #include <QtQuickTemplates2/private/qquickapplicationwindow_p.h>
 #include <QtQuickTemplates2/private/qquickcombobox_p.h>
+#include <QtQuickTemplates2/private/qquickdialog_p.h>
 #include <QtQuickTemplates2/private/qquickoverlay_p.h>
 #include <QtQuickTemplates2/private/qquickpopup_p.h>
 #include <QtQuickTemplates2/private/qquickbutton_p.h>
@@ -83,6 +85,12 @@ private slots:
     void enabled();
     void orientation_data();
     void orientation();
+    void qquickview();
+    void disabledPalette();
+    void disabledParentPalette();
+    void countChanged();
+    void toolTipCrashOnClose();
+    void setOverlayParentToNull();
 };
 
 void tst_QQuickPopup::initTestCase()
@@ -488,7 +496,7 @@ void tst_QQuickPopup::closePolicy()
     QTRY_VERIFY(popup->isOpened());
 
     // press outside popup and its parent
-    QTest::mousePress(window, Qt::LeftButton, Qt::NoModifier, QPoint(1, 1));
+    QTest::mousePress(window, Qt::LeftButton, Qt::NoModifier, QPoint(1, 1), 50);
     if (closePolicy.testFlag(QQuickPopup::CloseOnPressOutside) || closePolicy.testFlag(QQuickPopup::CloseOnPressOutsideParent))
         QTRY_VERIFY(!popup->isVisible());
     else
@@ -1060,6 +1068,167 @@ void tst_QQuickPopup::orientation()
     QCOMPARE(popup->popupItem()->position(), position);
 }
 
+void tst_QQuickPopup::qquickview()
+{
+    QQuickView view;
+    view.setObjectName("QQuickView");
+    view.resize(400, 400);
+    view.setSource(testFileUrl("dialog.qml"));
+    QVERIFY(view.status() != QQuickView::Error);
+    view.contentItem()->setObjectName("QQuickViewContentItem");
+    view.show();
+
+    QQuickDialog *dialog = view.rootObject()->property("dialog").value<QQuickDialog*>();
+    QVERIFY(dialog);
+    QTRY_COMPARE(dialog->property("opened").toBool(), true);
+
+    dialog->close();
+    QTRY_COMPARE(dialog->property("visible").toBool(), false);
+
+    // QTBUG-72746: shouldn't crash on application exit after closing a Dialog when using QQuickView.
+}
+
+// TODO: also test it out without setting enabled directly on menu, but on a parent
+
+// QTBUG-73447
+void tst_QQuickPopup::disabledPalette()
+{
+    QQuickApplicationHelper helper(this, "disabledPalette.qml");
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QQuickPopup *popup = window->property("popup").value<QQuickPopup*>();
+    QVERIFY(popup);
+
+    QSignalSpy popupEnabledSpy(popup, SIGNAL(enabledChanged()));
+    QVERIFY(popupEnabledSpy.isValid());
+    QSignalSpy popupPaletteSpy(popup, SIGNAL(paletteChanged()));
+    QVERIFY(popupPaletteSpy.isValid());
+
+    QSignalSpy popupItemEnabledSpy(popup->popupItem(), SIGNAL(enabledChanged()));
+    QVERIFY(popupItemEnabledSpy.isValid());
+    QSignalSpy popupItemPaletteSpy(popup->popupItem(), SIGNAL(paletteChanged()));
+    QVERIFY(popupItemPaletteSpy.isValid());
+
+    QPalette palette = popup->palette();
+    palette.setColor(QPalette::Active, QPalette::Base, Qt::green);
+    palette.setColor(QPalette::Disabled, QPalette::Base, Qt::red);
+    popup->setPalette(palette);
+    QCOMPARE(popupPaletteSpy.count(), 1);
+    QCOMPARE(popupItemPaletteSpy.count(), 1);
+    QCOMPARE(popup->background()->property("color").value<QColor>(), Qt::green);
+
+    popup->setEnabled(false);
+    QCOMPARE(popupEnabledSpy.count(), 1);
+    QCOMPARE(popupItemEnabledSpy.count(), 1);
+    QCOMPARE(popupPaletteSpy.count(), 2);
+    QCOMPARE(popupItemPaletteSpy.count(), 2);
+    QCOMPARE(popup->background()->property("color").value<QColor>(), Qt::red);
+}
+
+void tst_QQuickPopup::disabledParentPalette()
+{
+    QQuickApplicationHelper helper(this, "disabledPalette.qml");
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QQuickPopup *popup = window->property("popup").value<QQuickPopup*>();
+    QVERIFY(popup);
+
+    QSignalSpy popupEnabledSpy(popup, SIGNAL(enabledChanged()));
+    QVERIFY(popupEnabledSpy.isValid());
+    QSignalSpy popupPaletteSpy(popup, SIGNAL(paletteChanged()));
+    QVERIFY(popupPaletteSpy.isValid());
+
+    QSignalSpy popupItemEnabledSpy(popup->popupItem(), SIGNAL(enabledChanged()));
+    QVERIFY(popupItemEnabledSpy.isValid());
+    QSignalSpy popupItemPaletteSpy(popup->popupItem(), SIGNAL(paletteChanged()));
+    QVERIFY(popupItemPaletteSpy.isValid());
+
+    QPalette palette = popup->palette();
+    palette.setColor(QPalette::Active, QPalette::Base, Qt::green);
+    palette.setColor(QPalette::Disabled, QPalette::Base, Qt::red);
+    popup->setPalette(palette);
+    QCOMPARE(popupPaletteSpy.count(), 1);
+    QCOMPARE(popupItemPaletteSpy.count(), 1);
+    QCOMPARE(popup->background()->property("color").value<QColor>(), Qt::green);
+
+    // Disable the overlay (which is QQuickPopupItem's parent) to ensure that
+    // the palette is changed when the popup is indirectly disabled.
+    popup->open();
+    QTRY_VERIFY(popup->isOpened());
+    QVERIFY(QMetaObject::invokeMethod(window, "disableOverlay"));
+    QVERIFY(!popup->isEnabled());
+    QVERIFY(!popup->popupItem()->isEnabled());
+    QCOMPARE(popup->background()->property("color").value<QColor>(), Qt::red);
+    QCOMPARE(popupEnabledSpy.count(), 1);
+    QCOMPARE(popupItemEnabledSpy.count(), 1);
+    QCOMPARE(popupPaletteSpy.count(), 2);
+    QCOMPARE(popupItemPaletteSpy.count(), 2);
+
+    popup->close();
+    QTRY_VERIFY(!popup->isVisible());
+}
+
+void tst_QQuickPopup::countChanged()
+{
+    QQuickApplicationHelper helper(this, "countChanged.qml");
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QQuickComboBox *comboBox = window->property("comboBox").value<QQuickComboBox*>();
+    QVERIFY(comboBox);
+    QCOMPARE(window->property("count").toInt(), 1);
+
+    QVERIFY(window->setProperty("isModel1", false));
+    QTRY_COMPARE(window->property("count").toInt(), 2);
+}
+
+// QTBUG-73243
+void tst_QQuickPopup::toolTipCrashOnClose()
+{
+    QQuickApplicationHelper helper(this, "toolTipCrashOnClose.qml");
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    // TODO: Using ignoreMessage() fails in CI with macOS for release builds,
+    // so for now we let the warning through.
+//    QTest::ignoreMessage(QtWarningMsg, "ShaderEffectSource: 'recursive' must be set to true when rendering recursively.");
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QTest::mouseMove(window, QPoint(window->width() / 2, window->height() / 2));
+    QTRY_VERIFY(window->property("toolTipOpened").toBool());
+
+    QVERIFY(window->close());
+    // Shouldn't crash.
+}
+
+void tst_QQuickPopup::setOverlayParentToNull()
+{
+    QQuickApplicationHelper helper(this, "toolTipCrashOnClose.qml");
+
+    QQuickWindow *window = helper.window;
+    window->show();
+    // TODO: Using ignoreMessage() fails in CI with macOS for release builds,
+    // so for now we let the warning through.
+//    QTest::ignoreMessage(QtWarningMsg, "ShaderEffectSource: 'recursive' must be set to true when rendering recursively.");
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QVERIFY(QMetaObject::invokeMethod(window, "nullifyOverlayParent"));
+
+    QTest::mouseMove(window, QPoint(window->width() / 2, window->height() / 2));
+    QTRY_VERIFY(window->property("toolTipOpened").toBool());
+
+    QVERIFY(window->close());
+    // While nullifying the overlay parent doesn't make much sense, it shouldn't crash.
+}
+
 QTEST_QUICKCONTROLS_MAIN(tst_QQuickPopup)
 
 #include "tst_qquickpopup.moc"
